# Copyright (C) 2015 Red Hat
#
# This file is part of relval.
#
# wikitcms is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Adam Williamson <awilliam@redhat.com>
#
"""This file contains functions that help implement the 'size_check'
sub-command of relval, which checks the sizes of images and reports
the results.
"""

import fedfind.helpers
import fedfind.release

# https://fedoraproject.org/wiki/QA:Testcase_Mediakit_ISO_Size
# https://fedoraproject.org/wiki/Releases/32/Spins
# https://fedoraproject.org/wiki/Releases/32/ReleaseBlocking
MAXSIZES = {
    'netinst': 734003200,
    'boot': 734003200,
    'cloud': 734003200,
    'cloud_base': 734003200,
    'docker_base': 734003200,
    'raw-xz': 4000000000,
    'kde_live': 4700000000,
    'workstation_live': 2000000000,
    'design_suite_live': 4000000000,
    'lxde_live': 1500000000,
    'mate_live': 3000000000,
    'security_live': 2000000000,
    'soas_live': 734003200,
    'xfce_live': 2000000000,
    'astronomy_kde_live': 4000000000,
    'cinnamon_live': 2600000000,
    'jam_kde_live': 4000000000,
    'games_live': 16000000000,
    'python_classroom_live': 2000000000,
    'robotics_live': 4000000000,
    'scientific_kde_live': 3300000000,
    'lxqt_live': 1400000000,
    'comp_neuro_live': 4000000000,
    'xfce_raw-xz': 8000000000,
}

STDTYPES = ('netinst', 'boot', 'dvd', 'live', 'dvd-ostree', 'raw-xz')

def size_check(ffrel):
    """Do the thing! Check the things! CHECK ALL THE THINGS"""
    results = dict()
    # there's also Container and Silverblue, but we don't have any
    # known target sizes there as of 2019-09
    for variant in ("Everything", "Server", "Workstation", "Cloud", "Spins", "Labs"):
        print("Checking {0}...".format(variant))
        fails = list()
        imgs = [img for img in ffrel.all_images if img['variant'] == variant]
        imgs = [img for img in imgs if img['type'] in STDTYPES]
        if variant in ("Spins", "Labs"):
            imgs = [img for img in imgs if img['type'] in ("live", "raw-xz")]
        if not imgs:
            print("No images found!")
            continue

        for img in imgs:
            maxsize = 4700000000
            maxsize = MAXSIZES.get(img['type'], maxsize)
            maxsize = MAXSIZES.get(img['subvariant'].lower(), maxsize)
            maxsize = MAXSIZES.get('_'.join((img['subvariant'].lower(), img['type'])), maxsize)
            url = '/'.join((ffrel.location, img['path']))
            imgsize = fedfind.helpers.get_size(url)
            if imgsize > maxsize:
                fails.append((img, imgsize, maxsize))

        if fails:
            results[variant] = fails
            for fail in fails:
                filename = fail[0]['path'].split('/')[-1]
                print("FAIL: {0}, size {1}, max {2}".format(filename,
                                                            fail[1], fail[2]))
        else:
            results[variant] = 'pass'
            print("PASS!")
    return results
